﻿using Microsoft.Extensions.Logging;
using PrecastConcretePlantContracts.BindingModels;
using PrecastConcretePlantContracts.BusinessLogicsContracts;
using PrecastConcretePlantContracts.SearchModels;
using PrecastConcretePlantContracts.StoragesContracts;
using PrecastConcretePlantContracts.ViewModels;
using PrecastConcretePlantDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PrecastConcretePlantBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;
        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }
        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. ShopName:{ShopName}.Id:{ Id} ",
                model?.Name, model?.Id);
            var list = (model == null) ? _shopStorage.GetFullList() :
            _shopStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ShopName:{ShopName}.Id:{ Id}",
                model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            model.Reinforcedies = new();
            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(ShopBindingModel model)
        {
            CheckModel(model, false);
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина",
                nameof(model.Name));
            }
            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина",
                nameof(model.Name));
            }
            _logger.LogInformation("Shop. ShopName:{0}.Address:{1}. Id: {2}",
                model.Name, model.Address, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.Name
            });
            if (element != null && element.Id != model.Id && element.Name == model.Name)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

        public bool AddReinforced(ShopSearchModel model, IReinforcedModel reinforced, int count)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (count <= 0)
            {
                throw new ArgumentException("Количество добавляемого изделия должно быть больше 0", nameof(count));
            }
            _logger.LogInformation("AddReinforcedInShop. ShopName:{ShopName}.Id:{ Id}",
                model.Name, model.Id);
            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("AddReinforcedInShop element not found");
                return false;
            }
            _logger.LogInformation("AddReinforcedInShop find. Id:{Id}", element.Id);


            if (element.Reinforcedies.TryGetValue(reinforced.Id, out var pair))
            {
                element.Reinforcedies[reinforced.Id] = (reinforced, count + pair.Item2);
                _logger.LogInformation(
                    "AddReinforcedInShop. Has been added {count} {reinforced} in {ShopName}",
                    count, reinforced.ReinforcedName, element.Name);
            }
            else
            {
                element.Reinforcedies[reinforced.Id] = (reinforced, count);
                _logger.LogInformation(
                    "AddReinforcedInShop. Has been added {count} new Reinforced {reinforced} in {ShopName}",
                    count, reinforced.ReinforcedName, element.Name);
            }
            _shopStorage.Update(new()
            {
                Id = element.Id,
                Address = element.Address,
                Name = element.Name,
                DateOpening = element.DateOpening,
                Reinforcedies = element.Reinforcedies
            });
            return true;
        }
    }
}
